--============================================================================
-- Design units : general
-- File name    : getStream.vhd
--
-- Description  : 
--
--
-- Uses         : None
--
-- Author       : Klaus Ruzicka, Zurich, Switzerland
--
--============================================================================
-- Revision list
-- Version  Author  Date      Changes
--
-- 1.0      Rkl     14/01/99  New design
--============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use STD.TEXTIO.all;

entity getRLLstream is
-- This entity reads al RLL (run length limited) data file an produces a bit
-- stream. For changing file name and options see the file/constant declaration
-- in the architecture of the entity. The data file can be devided into two
-- parts:
-- 1) declaraions
--	0 optional comments
--	s optional comments: start bit stream with '1' instead of '0', s is a 
--			negative number defined by constant switch, 
--			default is s=-9
--	n code_stream 0 optional comments: define a macro, n is a negative 
--			number, the range is defined by constant macroCnt, the
--			maximum length of the code stream is defined by 
--			constant macroLen. In the code stream other macros may
--			be used. A recursive use (infinite loop) is also 
--			possible: n code_stream n 0 optional comments.
--			A macro starts always with a '0' unless you use the s
--			macro: n s code_stream 0 optional comments
--			A macro may use always only one line, and has to be 
--			finished with a '0'.
-- 2) regular code
-- The section starts with a positive number. Afterwards macros can be used by
-- putting the macro number (negative) into the code stream.
  port ( clk: in  std_logic;
 	 Q:   out std_logic);
end getRLLstream;

architecture behav of getRLLstream is
--file rll: 	     text is in "filename";
  file rll: 	     text is in "RLLstream.txt"; -- change file name ONLY
  constant edge:     std_logic := '1'; -- 1=rising, 0=falling edge active
  constant switch :  integer :=  -9;   -- value to start sequence with 1
  constant macroCnt: integer :=   3;   -- max count of macros: 1 to -(switch+1)
  constant macroLen: integer := 255;   -- max no of values in macro 
 				       -- (including trailing 0) 
  constant macroNes: integer :=   3;   -- max nesting level of macros
 				       -- (recursive ones excluded)
-- DON'T CHANGE ANYTHING BELOW THIS LINE -------------------------------------
  type tmacro is array (1 to macroCnt,1 to macroLen) of integer;
  type tRmacro is record
 		  mac : integer;	-- which macro
 		  pos : integer;	-- position in this macro
 		end record;
  type tNmacro is array (1 to macroNes) of tRmacro;

begin
  nextValue: process(clk)
  variable first : boolean := true;	-- need to initialize
  variable last  : boolean := false;	-- no data left
  variable textline : line;		-- read next data line
  variable cnt   : integer := 0; 	-- repeat state cnt times
  variable act   : integer;		-- actual repetition number
  variable macro : tmacro;		-- contents of macros
  variable Q1    : std_logic;		-- intermediate result
  variable actM  : integer;		-- actual macro, 0 if no
  variable mCnt  : integer;		-- position in actual macro
  variable nest  : tNmacro;		-- info for nested macros
  variable nCnt  : integer;		-- macro nesting level
  begin
    if clk'event and clk=edge then
      if first then		-- declarative part
        Q1:='0';
        first := false;
        while cnt <= 0 and not endFile(rll) loop
          readline(rll,textline);
          read(textline,cnt);
          if cnt < 0 then	-- ignore commands and comments
            if cnt < -1*macroCnt then
              if cnt=switch then	-- start with '1'
                Q1:='1';
              else		-- illegal macro number
                assert false report "getBitStream: MACRO IGNORED - increase macroCnt!" 
 							severity warning;
              end if;
            else		-- macro
              act:=1;
              mCnt:=1;
              while act/=0 loop
                read(textline,act);
                if mCnt=macroLen then
                  assert act=0 report "getBitStream: MACRO TOO LONG - CUTTED - increase macroLen!" 
 							severity warning;
                  act:=0; 
                end if;
                macro(abs(cnt),mCnt):=act;
                mCnt:=mCnt+1;
              end loop;
 	    end if;
          end if;          
        end loop;
        act:=1;
        actM:=0;
        nCnt:=0;
        if cnt=0 then
          last:=true;
          assert false report "getBitStream: NO DATA IN FILE!" severity warning;
        end if;
      else -- regular part-----------------------------------------
        if act<cnt then  -- in progress
          act:=act+1;
        elsif not last then -- load   
          Q1:=not Q1;
          first:=true; -- jump into while loop
          while first or cnt<=0 loop
            first:=false; -- jump done, reset to stay in regular part
            if cnt>0 then
              if actM=0 then	-- normal file read
                read(textline,cnt);
              else		-- macro is in progress
                mCnt:=mCnt+1;
                cnt:=macro(actM,mCnt);
              end if;
            end if;
            if cnt<0 then	-- we start with a macro or an error
              if cnt>=-1*macroCnt then 			-- macro
                if actM/=0 and actM+cnt/=0 then 
                  -- called from macro (ignore recursive calls)
                  if nCnt<macroNes then                
                    nCnt:=nCnt+1;
                    nest(nCnt).mac:=actM;
                    nest(nCnt).pos:=mCnt;
                  else
                    assert false report "getBitStream: MACRO NESTING LEVEL OVERFLOW - increase macroNes!" 
 							severity error;
                  end if;
                end if;
                actM:=abs(cnt);
                mCnt:=1;
                cnt:=macro(actM,mCnt); --read 1st value
                if cnt=switch then	-- start macro with
                  Q1:='1';		-- '1'
                  mCnt:=mCnt+1;
                  cnt:=macro(actM,mCnt);
                else
                  Q1:='0';		-- '0' (default)
                end if;
              else 					-- error
                assert false report "getBitStream: UNKNOWN MACRO IGNORED!" 
 							severity warning;
              end if;
            end if;
 
            if cnt=0 then	-- new line / macro finished
 	      -- everything after the 0 is treated as comment
              if endFile(rll) then
                last := true;
                cnt  := 1; -- leave loop
                assert false report "getBitStream: END OF FILE!" 
 		 					severity warning;
              else
                if actM=0 then 		-- line done, get next line from file
                  readline(rll,textline); 
                end if; 
 		-- read next value
                if nCnt>0 then		-- we came from macro
                  actM:=nest(nCnt).mac;
                  mCnt:=nest(nCnt).pos+1;
                  nCnt:=nCnt-1;
                  cnt:=macro(actM,mCnt);
                else			-- regular from file
                  read(textline,cnt);
                  actM:=0;
                end if;
              end if; -- eof
            end if; -- if cnt=0
          end loop;
          act:=1;
        end if;  -- in progress / load
      end if;  -- declaration/regular part
    end if; -- clocked process
    Q <= Q1;
  end process; -- nextValue

end behav; -- getRLLstream
--============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity pns7 is
-- Create pseudo noise sequence of length 2^7-1
  port ( clk: in  std_logic;
	 Q:   out std_logic);
end pns7;

architecture behav of pns7 is
  constant edge:     std_logic := '1'; -- 1=rising, 0=falling edge active
begin
  pns:process(clk)
  variable reg : std_logic_vector ( 6 downto 0 ) := (6=>'1', others=>'0');
  variable first: std_logic;
  begin
    if clk'event and clk=edge then
      first:=reg(0) xor reg(3);
      reg:=shr(reg,"1");
      reg(6):=first;
      Q <= reg(0); 
    end if; -- clocked process
  end process; -- pns
end behav; -- pns7
--============================================================================
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity pns9 is
-- Create pseudo noise sequence of length 2^9-1
  port ( clk: in  std_logic;
	 Q:   out std_logic);
end pns9;

architecture behav of pns9 is
  constant edge:     std_logic := '1'; -- 1=rising, 0=falling edge active
begin
  pns:process(clk)
  variable reg : std_logic_vector ( 8 downto 0 ) := (8=>'1', others=>'0');
  variable first: std_logic;
  begin
    if clk'event and clk=edge then
      first:=reg(0) xor reg(4);
      reg:=shr(reg,"1");
      reg(8):=first;
      Q <= reg(0); 
    end if; -- clocked process
  end process; -- pns
end behav; -- pns9
-- ===========================================================================
-- end of file
-- ===========================================================================
